<?php
/************************************************************************
 * MXChange v0.2.1                                    Start: 03/10/2009 *
 * ===============                              Last change: 03/10/2009 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : inc-functions.php                                *
 * -------------------------------------------------------------------- *
 * Short description : Special functions for handling include files     *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Spezielle Funktionen fuer Include-Dateien        *
 * -------------------------------------------------------------------- *
 * $Revision:: 999                                                    $ *
 * $Date:: 2009-03-10 17:24:54 +0100 (Tue, 10 Mar 2009)               $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author:: quix0r                                                   $ *
 * Needs to be in all Files and every File needs "svn propset           *
 * svn:keywords Date Revision" (autoprobset!) at least!!!!!!            *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2008 by Roland Haeder                           *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	$INC = substr(dirname(__FILE__), 0, strpos(dirname(__FILE__), '/inc') + 4) . '/security.php';
	require($INC);
}

// Check if our config file is writeable or not
function isIncludeWriteable ($inc) {
	// Generate FQFN
	$FQFN = sprintf("%sinc/%s.php", constant('PATH'), $inc);

	// Abort by simple test
	if ((isFileReadable($FQFN)) && (!is_writeable($FQFN))) {
		return false;
	} // END - if

	// Test write-access on directory
	return is_writeable(dirname($FQFN));
}

// Reads a directory with PHP files in and gets only files back
function getArrayFromDirectory ($baseDir, $prefix, $includeDirs = false, $addBaseDir = true, $excludePattern = '@(\.|\.\.)$@') {
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "baseDir={$baseDir},prefix={$prefix} - Entered!");
	// Init includes
	$INCs = array();

	// Open directory
	$dirPointer = opendir(constant('PATH') . $baseDir) or app_die(__FUNCTION__, __LINE__, "Cannot read ".basename($baseDir)." path!");

	// Read all entries
	while ($baseFile = readdir($dirPointer)) {
		// Steps over this returned $baseFile-Name, when it matches the $excludePattern
		if (preg_match($excludePattern, $baseFile, $match)) {
			// These Lines are only for debugging!!
		    //$INC = $baseDir . '/' . $baseFile;
			//$FQFN = constant('PATH') . $INC;
			//echo '<pre>$baseDir:'.print_r($baseDir, true).'</pre>';
			//echo '<pre>$baseDir:'.print_r(constant('PATH') . $baseDir, true).'</pre>';
			//echo '<pre>constant(\'PATH\'):'.print_r(constant('PATH'), true).'</pre>';
			//echo '<pre>$FQFN:'.print_r($FQFN, true).'</pre>';
			continue;
		} // END - if

		// Construct include filename and FQFN
		$INC = $baseDir . '/' . $baseFile;
		$FQFN = constant('PATH') . $INC;

		// repalecment of // to / is needed, whenn $baseDir is an emty String
		$FQFN = str_replace('//', '/', $FQFN);

		// Is this a valid reset file?
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "baseDir={$baseDir},prefix={$prefix},baseFile={$baseFile}");
		if (((isFileReadable($FQFN)) && (substr($baseFile, 0, strlen($prefix)) == $prefix) && (substr($baseFile, -4, 4) == ".php")) || (($includeDirs) && (isDirectory($FQFN)))) {
			// Remove both for extension name
			$extName = substr($baseFile, strlen($prefix), -4);

			// Try to find it
			$extId = GET_EXT_ID($extName);

			// Is the extension valid and active?
			if (($extId > 0) && (EXT_IS_ACTIVE($extName))) {
				// Then add this file
				//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, " Extension entry ".$baseFile." added.");
				$INCs[] = $INC;
			} elseif ($extId == 0) {
				// Add non-extension files as well
				//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, " Regular entry ".$baseFile." added.");
				if ($addBaseDir) {
					$INCs[] = $INC;
				} else {
					$INCs[] = $baseFile;
				}
			}
		} // END - if
	} // END - while

	// Close directory
	closedir($dirPointer);

	// Sort array
	asort($INCs);

	// Return array with include files
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, " - Left!");
	return $INCs;
}

// Init INC_POOL
function INIT_INC_POOL () {
	$GLOBALS['inc_pool'] = array();
}

// Setter for INC_POOL
function SET_INC_POOL ($includePool) {
	$GLOBALS['inc_pool'] = (array) $includePool;
}

// Getter for INC_POOL
function GET_INC_POOL () {
	return $GLOBALS['inc_pool'];
}

// Count INC_POOL
function COUNT_INC_POOL () {
	return count($GLOBALS['inc_pool']);
}

// Merge INC_POOL into given
function MERGE_INC_POOL ($includePool) {
	SET_INC_POOL(merge_array(GET_INC_POOL(), $includePool));
}

// Add single include file to INC_POOL
function ADD_INC_TO_POOL ($INC) {
	$GLOBALS['inc_pool'][] = (string) $INC;
}

// Remove an include file from INC_POOL
function REMOVE_INC_FROM_POOL ($INC) {
	// First look it up
	$key = array_search($INC, GET_INC_POOL());

	// Is it valid?
	if ($key !== false) {
		// Then remove it
		unset($GLOBALS['inc_pool'][$key]);

		// And sort the list
		asort($GLOBALS['inc_pool']);
	} // END - if
}

// [EOF]
?>
